#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2022 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys, argparse
import json
from http.server import BaseHTTPRequestHandler, HTTPServer

from broadway2 import Broadway
from broadway2 import get_phy_access

Json = {}

# HTTPRequestHandler class
class HTTPServer_RequestHandler(BaseHTTPRequestHandler):
    # send header for json-structure and allow access for http-server
    def _set_headers(self):
        # Send response status code
        self.send_response(200)
        self.send_header('Content-type', 'application/json')
        self.send_header('Access-Control-Allow-Origin', '*')
        self.end_headers()

    # do_head
    def do_HEAD(self):
        self._set_headers()
    # GET
    def do_GET(self):
        self._set_headers()

        # Send message back to client
        
        # Write content as utf-8 data
        self.wfile.write(bytes(Json, "UTF-8"))
        return

    # POST echoes the message adding a JSON field
    def do_POST(self):
        print('POST!')
        ctype, pdict = cgi.parse_header(self.headers.getheader('content-type'))
        print(ctype, pdict)
        # refuse to receive non-json content
        if ctype != 'application/json':
            self.send_response(400)
            self.end_headers()
            return
            
        # read the message and convert it into a python dictionary
        length = int(self.headers.getheader('content-length'))
        message = json.loads(self.rfile.read(length))
        
        print('POST message', message)
        
        
        # send the message back
        self._set_headers()
        self.wfile.write(json.dumps(message))

def run(ipAddress, portNumber):
    #print('starting server...')

    # Server settings from cmd-line args
    
    serverAddress = (ipAddress, portNumber)
    httpd = HTTPServer(serverAddress, HTTPServer_RequestHandler)
    #print('running server...')
    httpd.serve_forever()


if __name__=='__main__':
    # default startup (no args given)
    ip = 'localhost'
    port = 1234

    if len(sys.argv) > 1:
        parser = argparse.ArgumentParser(description='HTTP Server')
        parser.add_argument('port', type=int, help='Listening port for HTTP Server', default=1234)
        parser.add_argument('ip', help='HTTP Server IP', default='localhost')
        args = parser.parse_args()
        ip = args.ip
        port = args.port

    base = Broadway()

    adaDict = {}
    for adapter in base.enumerate():
        adaDict[adapter.Inst] = {'FriendlyName': adapter.FriendlyName, 'SerialNumber': adapter.SerialNumber, 'USBProductId': hex(adapter.UsbProductId), 'MACAddress': adapter.MACAddress}
        
        #str += 'USB-Inst: {0} FriendlyName: {1} SerialNumber: {2}\n'.format(adapter.Inst, adapter.FriendlyName, adapter.SerialNumber)

    # fill-up global Json variable
    Json = '{0}'.format(json.dumps(adaDict))

    try:
        print('Starting JSON-Server on {0}:{1}'.format(ip, port))
        print('Press Ctrl-C to exit!')
        run(ip, port)
    except KeyboardInterrupt:
        pass

