#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import sys, os
import binascii
import platform
import time, datetime
import threading, queue
import binascii

from broadway2 import broadway_api, Broadway, RawReceiver, RawRxHeaderTypes, EthFrame, RawRxFrame


def main(args):
    
    # create adapter singleton
    base = Broadway()

    print('Connected Devices: ')
    print('='*25)

    aList = base.get_raw_adapters()


    if len(aList) > 1:
        cnt = 1
        for a in aList:
            print('[ {0:2} ]: {1} / {2}'.format(cnt, a[1], a[0]))
            cnt += 1
        try:
            sel = int(input('Enter Number to use:'))
        except:
            sel = 0
        
    else:
        sel = 0


    inst, friendlyName = aList[sel-1]

    adapter = base.get_adapter(inst)

         
    if adapter != None:
        devTx = base.open_raw_tx(adapter.Inst)
        devRx = base.open_raw_rx(adapter.Inst, RawRxHeaderTypes.HeaderType2)
        print('Raw Rx/Tx Device opened: {0}'.format(friendlyName))

        rawReceiver = RawReceiver(devRx)

        rawReceiver.enable()

        print('Receiver running... Ctrl-C to exit')

        try:
            while 1:
                # read_block with max. 500 frames and timeout after 500 ms
                result = rawReceiver.read_block(1000, 500)
                # returns array of tuples (timestamp, seqCounter, Ethernet-Frame)
                for res in result:
                    ethRaw = RawRxFrame(1, res)
                    # print(' -> {0:8} {1}.{2}'.format(ethRaw.SequenceCnt, ethRaw.TimestampSec, ethRaw.TimestampNano))
                    eth = ethRaw.EthFrame
                    # print('  {0} {1}'.format(str(eth), eth.Data[0:16]))
                    
                    # prepare echo frame, build out of received data
                    
                    sendData = '{0:012X}'.format(eth.TgtMac) + '{0:012X}'.format(eth.SrcMac) + '{0:04X}'.format(len(eth.Data)) 
                    binData = binascii.a2b_hex(sendData) + binascii.a2b_hex(eth.Data)

                    res = broadway_api.wait_for_tx(devTx, 1000)
                    if not res:
                        print('waitfor tx failed!')

                    frameId1 = broadway_api.submit_raw_frame(devTx, binData)

                    res = broadway_api.send_raw_frames(devTx)
            
                    res = broadway_api.wait_for_raw_tx_completion(devTx, frameId1, 1000)
                    
                    hwTimeStamp1 = broadway_api.get_raw_tx_timestamp(devTx, frameId1)

                    sendData = 'FFFFFFFFFFFFFC0011223344' + '0030' + '{0:016X}'.format(ethRaw.TimestampSec) + '{0:08X}'.format(ethRaw.TimestampNano) + '{0:016X}'.format(hwTimeStamp1[0]) + '{0:08X}'.format(hwTimeStamp1[1])
                    binData = binascii.a2b_hex(sendData)

                    frameId2 = broadway_api.submit_raw_frame(devTx, binData)

                    res = broadway_api.send_raw_frames(devTx)

                    res = broadway_api.wait_for_raw_tx_completion(devTx, frameId2, 1000)

                    hwTimeStamp2 = broadway_api.get_raw_tx_timestamp(devTx, frameId2)

                    print('sent out 2 frames: {0} / {1}'.format(frameId1, frameId2))
                    diffSec = hwTimeStamp2[0] - hwTimeStamp1[0]
                    diffNano = hwTimeStamp2[1] - hwTimeStamp1[1]
                    print('TimeDiff: {0} s {1} ms'.format(diffSec, diffNano/1e6))

                    
        except KeyboardInterrupt:

            rawReceiver.disable()

            base.close(devRx)
            base.close(devTx)

if __name__ == "__main__":
    main(sys.argv)

